#include "board.h"

static const SDL_Color white = {255,255,255};
static const SDL_Color black = {0,0,0};

void Game::draw_info(SDL_Surface *surface, const Thing &t, Text *text) {
  SDL_Rect re;
  re.x = SCREENWIDTH-INFOWIDTH+(INFOWIDTH-GRID)/2;
  re.y = 120;
  re.h = re.w = GRID;
  draw_thing(&re,surface,t,text);
  text->drawtext_cc(t.description(1),surface,re.x+GRID/2,150+GRID,black,text->small);
  text->drawtext_cc(t.description(2),surface,re.x+GRID/2,180+GRID,black,text->tiny);
  if (t.istower()) {
    re.x = SCREENWIDTH-INFOWIDTH+10;
    re.w = INFOWIDTH-20;
    re.y = 230;
    re.h = 25;
    SDL_FillRect(surface,&re,SDL_MapRGB(surface->format,0,0,80));
    re.x += 1;
    re.y += 1;
    re.h -= 2;
    re.w = (INFOWIDTH-22)*t.hp/t.initial_hp();
    SDL_FillRect(surface,&re,SDL_MapRGB(surface->format,0,0,255));
    re.x = SCREENWIDTH-INFOWIDTH+(INFOWIDTH-GRID)/2;
    char msg[80];
    snprintf(msg,80,"HP: %i/%i",t.hp,t.initial_hp());
    text->drawtext_c(msg,surface,re.x+GRID/2,231,white,text->small);
    if (t.hp<t.initial_hp()) {
        snprintf(msg,80,"[Y] to repair (-%i gold)",t.sell_value()/10);
        text->drawtext_c(msg,surface,re.x+GRID/2,260,white,text->tiny);
    }
    snprintf(msg,80,"Range: %.2f",sqrt(t.range()));
    text->drawtext_c(msg,surface,re.x+GRID/2,290,white,text->tiny);
    snprintf(msg,80,"Damage/second: %i",t.dmg()*FPS);
    text->drawtext_c(msg,surface,re.x+GRID/2,310,white,text->tiny);
    switch(t.mode) {
	case 0: snprintf(msg,80,"Mode 1: normal"); break;
	case 1: snprintf(msg,80,"Mode 2: bigger range"); break;
	case 2: snprintf(msg,80,"Mode 3: weakest 1st"); break;
	case 3: snprintf(msg,80,"Mode 4: strongest 1st"); break;
	case 4: snprintf(msg,80,"Mode 5: fastest 1st"); break;
	case 5: snprintf(msg,80,"Mode 6: aggressive 1st"); break;
	default: snprintf(msg,80,"Mode: unknown");
    }
    text->drawtext_c(msg,surface,re.x+GRID/2,330,white,text->tiny);
    snprintf(msg,80,"(trigger buttons to change)");
    text->drawtext_c(msg,surface,re.x+GRID/2,350,black,text->tiny);
    snprintf(msg,80,"[X] to destroy (+%i gold)",t.sell_value());
    text->drawtext_c(msg,surface,re.x+GRID/2,380,white,text->tiny);
    filledCircleRGBA(surface,level->x+t.c*GRID+GRID/2,level->y+t.r*GRID+GRID/2,sqrt(t.range())*GRID,255,0,0,50);
    aacircleRGBA(surface,level->x+t.c*GRID+GRID/2,level->y+t.r*GRID+GRID/2,sqrt(t.range())*GRID,255,100,100,150);
  }
  if (canbuild()) {
    text->drawtext_c("[A] Obstacle (-50 gold)",surface,re.x+GRID/2,400,white,text->tiny);
    text->drawtext_c("[B] Shooter (-50 gold)",surface,re.x+GRID/2,420,white,text->tiny);
  } else if (t.can_upgrade()) {
    char msg[80];
    snprintf(msg,80,"[B] to upgrade (-%i gold)",t.upgrade_cost());
    text->drawtext_c(msg,surface,re.x+GRID/2,400,white,text->tiny);
  }


}
void Game::draw_thing(SDL_Rect *re, SDL_Surface *surface, const Thing &t, Text *text) {
  SDL_Rect tre;
  if (t.type < 32 || t.type > 127) return; // should not happen
  tre.x = GRID*(t.type % 8);
  tre.y = GRID*(t.type/8 - 4);
  tre.w = tre.h = GRID;
  if (SDL_BlitSurface(tiles,&tre,surface,re)) throw Exiter("meh");
  if (t.type == '*') {
        char msg[8];
        snprintf(msg,8,"%i",pleft);
        text->drawtext_cc(msg,surface,re->x+GRID/2,re->y+GRID/2,white,text->big);
  }
}
void Game::draw_selection(SDL_Rect &re, SDL_Surface *surface) {
    re.x -= 2;
    re.y -= 2;
    re.w += 4;
    re.h += 4;
    roundedBoxRGBA(surface,re.x+re.w,re.y,re.x,re.y+re.h,8,250,250,200,100);
    roundedRectangleRGBA(surface,re.x+re.w,re.y,re.x,re.y+re.h,8,255,255,0,150);
}

void Game::draw_effect(SDL_Surface *surface, const Effect &e) {
  int x1=e.x1;
  int y1=e.y1;
  int x2=e.x2;
  int y2=e.y2;
  if (x1>x2) {  // thickLineRGBA seems to have a bug
        x1=e.x2;
        y1=e.y2;
        x2=e.x1;
        y2=e.y1;
  }
  int bx=e.x2;
  int by=e.y2;
  if (e.bs > 1) {
        bx = (e.x2*(clock%(e.bs))+e.x1*((e.bs-1)-(clock%(e.bs))))/(e.bs-1);
        by = (e.y2*(clock%(e.bs))+e.y1*((e.bs-1)-(clock%(e.bs))))/(e.bs-1);
  }
  switch(e.type) {
     case 'T':
//        thickLineRGBA(surface,x1,y1,x2,y2,2,55+50*(clock%5),50*(clock%5),20*(clock%5),100+(10*clock%16));
//        filledCircleRGBA(surface,e.x2,e.y2,5+(e.size-5)*(clock%5)/4,250,200,0,60);
        filledCircleRGBA(surface,bx,by,e.size,250,250,0,250*(clock%(e.bs))/(e.bs-1));
        break;
     case 'U':
        filledCircleRGBA(surface,bx,by,e.size,250,150,50,55+200*(clock%(e.bs))/(e.bs-1));
        break;
     case 'V':
        filledCircleRGBA(surface,bx,by,e.size,250,100,150,55+200*(clock%(e.bs))/(e.bs-1));
        break;
     case 'Y':
        thickLineRGBA(surface,x1,y1,x2,y2,3,50*(clock%6),50*(clock%5),20*(clock%5),200);
        filledCircleRGBA(surface,e.x2,e.y2,5+(e.size-5)*(clock%5)/4,250,100,0,80);
        break;
     case 'X':
        thickLineRGBA(surface,x1,y1,x2,y2,6,250*(clock%2),80*(clock%4),200*(clock%2),100);
        thickLineRGBA(surface,x1,y1,x2,y2,2,250*(clock%2),220*(clock%2),200*(clock%2),200);
        filledCircleRGBA(surface,e.x2,e.y2,5+(e.size-5)*(clock%5)/4,250,0,0,100);
        break;
     case 'E':
        filledCircleRGBA(surface,bx,by,e.dmg,0,250,200,200);
        break;
     case 'B':
        aacircleRGBA(surface,e.x1,e.y1,5+(e.size-5)*(clock%11)/10,250,0,200,200);
        filledCircleRGBA(surface,bx,by,e.dmg,250,0,200,200);
        break;
     default:
        break;
  }
}
void Game::draw_enemy(SDL_Rect *re, SDL_Surface *surface, const Enemy &e) {
  SDL_Rect r = *re;
  int minsize = GRID - GRID*e.hp/e.maxhp;
  if (minsize%2) minsize--;
  if (minsize > GRID-8) minsize = GRID-8;
  r.x += minsize/2;
  r.y += minsize/2;
  r.w -= minsize;
  r.h -= minsize;
  switch(e.type) {
     case 'M':
        boxRGBA(surface,r.x+r.w,r.y,r.x,r.y+r.h,255,0,0,200);
        r.x += 3;
        r.y += 3;
        r.w -= 6;
        r.h -= 6;
        boxRGBA(surface,r.x+r.w,r.y,r.x,r.y+r.h,255,200,0,200);
        break;
     case 'B':
        roundedBoxRGBA(surface,r.x+r.w,r.y,r.x,r.y+r.h,12,255,200,0,150);
        r.x += 4;
        r.y += 4;
        r.w -= 8;
        r.h -= 8;
        if (r.w>2) filledCircleRGBA(surface,r.x+r.w/2,r.y+r.h/2,r.w/2,255,0,200,150);
        break;
     case 'S':
        boxRGBA(surface,r.x+r.w,r.y,r.x,r.y+r.h,255,100,0,200);
        r.x += 3;
        r.y += 3;
        r.w -= 6;
        r.h -= 6;
        boxRGBA(surface,r.x+r.w,r.y,r.x,r.y+r.h,200,255,50,255);
        break;
     case 'N':
        boxRGBA(surface,r.x+r.w,r.y,r.x,r.y+r.h,255,0,120,200);
        while (r.w>3) {
          rectangleRGBA(surface,r.x+r.w,r.y,r.x,r.y+r.h,150,0,130,200);
          r.x += 2;
          r.y += 2;
          r.w -= 4;
          r.h -= 4;
        }
        break;
     case 'U':
        rectangleRGBA(surface,r.x+r.w,r.y,r.x,r.y+r.h,(r.x%7)*40,(r.y%6)*50,(r.x%5)*60,250);
        while (r.w>6) {
          r.x += 1;
          r.y += 1;
          r.w -= 2;
          r.h -= 2;
          rectangleRGBA(surface,r.x+r.w,r.y,r.x,r.y+r.h,(r.x%7)*40,(r.y%6)*50,(r.x%5)*60,55+(r.y%5)*50);
        }
        break;
     case 'o':
        filledCircleRGBA(surface,r.x+r.w/2,r.y+r.h/2,r.w/2,150,250,0,150);
        r.w /= 2;
        r.h /= 2;
        filledCircleRGBA(surface,r.x+r.w,r.y+r.h,r.w/2,50,0,250,150);
        break;
     case 'O':
        filledCircleRGBA(surface,r.x+r.w/2,r.y+r.h/2,r.w/2,50,200,0,150);
        r.w /= 2;
        r.h /= 2;
        filledCircleRGBA(surface,r.x+r.w,r.y+r.h,r.w/2,0,50,250,150);
        break;
     case 'f':
        filledCircleRGBA(surface,r.x+r.w/2,r.y+r.h/2,r.w/2,150,250,100,180);
        r.w /= 2;
        r.h /= 2;
        filledCircleRGBA(surface,r.x+r.w,r.y+r.h,r.w/2,250,250,20*(clock%13),25*(clock%11));
        break;
     default:
        SDL_FillRect(surface,re,SDL_MapRGB(surface->format,255,0,255));
        break;
  }
}
void Game::draw_path(SDL_Surface *surface, int r, int c, bool swimmer, bool o) {
    int newr=0, newc=0;
    while(r != level->destr || c != level->destc) {
      if (o) {
	if (swimmer) {
	  newr = r + level->m(r,c).osdr;
	  newc = c + level->m(r,c).osdc;
	} else {
	  newr = r + level->m(r,c).odr;
	  newc = c + level->m(r,c).odc;
	}
      } else {
	if (swimmer) {
	  newr = r + level->m(r,c).sdr;
	  newc = c + level->m(r,c).sdc;
	} else {
	  newr = r + level->m(r,c).dr;
	  newc = c + level->m(r,c).dc;
	}
      }
	int d = - 2 + 2*swimmer + 4*o;
	int x1 = level->x + GRID*c + GRID/2 + d;
	int y1 = level->y + GRID*r + GRID/2 + d;
	int x2 = level->x + GRID*newc + GRID/2 + d;
	int y2 = level->y + GRID*newr + GRID/2 + d;
	aalineRGBA(surface,x1,y1,x2,y2,255*(1-swimmer),255,255*swimmer,150-o*100);
	r = newr;
	c = newc;
    }
}
void Game::draw_base(SDL_Surface *surface, Text *text) {
        if (SDL_BlitSurface(basebg,NULL,surface,NULL)) throw Exiter("strange");
        SDL_Rect re;
        if (levelnb > 0) {
        re.x = level->x-2;
        re.y = level->y-2;
        re.w = level->w+4;
        re.h = level->h+4;
        SDL_FillRect(surface,&re,SDL_MapRGB(surface->format,50,50,100));
        }
        re.w = GRID;
        re.h = GRID;
        for(int r=0; r<level->rows; r++) {
          for(int c=0; c<level->cols; c++) {
            re.x = level->x+c*GRID;
            re.y = level->y+r*GRID;
            draw_thing(&re,surface,level->m(r,c),text);
          }
        }
        if (levelnb > 0) {
          if (show_paths)
          for (int i=0; i<level->nb_e_locs; i++) {
            draw_path(surface,level->e_initr[i],level->e_initc[i],1,1);
            draw_path(surface,level->e_initr[i],level->e_initc[i],0,1);
            draw_path(surface,level->e_initr[i],level->e_initc[i],1,0);
            draw_path(surface,level->e_initr[i],level->e_initc[i],0,0);
          }
          text->drawtext_c(level->title,surface,320,10,white,text->small);
          draw_baseinfo(surface,text);
        }
        draw_baseinfo(surface,text);
        redraw_base = false;
}
void Game::draw_baseinfo(SDL_Surface *surface, Text *text) {
    if (levelnb == -1) {
        text->drawtext_c("Help",surface,700,400,white,text->huge);
        int x=230;
        int y;
        const char *name;
        const char *desc;
        char specs[80];
        for (unsigned int i=0; i<enemy.size(); i++) {
	    y = GRID*enemy[i].r;
	    switch(enemy[i].type) {
		case 'M': name = "Monster        "; desc="Most common enemy"; break;
		case 'N': name = "Big Monster    "; desc="Does damage"; break;
		case 'B': name = "Berserker Tank "; desc="Multi-attacks; goes straight to target"; break;
		case 'f': name = "Bird           "; desc="Can fly over towers"; break;
		case 'S': name = "Speedy Monster "; desc="Very fast"; break;
		case 'U': name = "Crazy Monster  "; desc="Moves unpredictably"; break;
		case 'o': name = "Fish           "; desc="Can swim"; break;
		case 'O': name = "Whale          "; desc="Can swim and does damage"; break;
		default: name = "unknown"; desc="unknown";
	    }
	    text->drawtext(name,surface,x,y-2,white,text->small);
	    text->drawtext(desc,surface,x+30,y+GRID/2+2,white,text->tiny);
	    snprintf(specs,80,"HP: %i   Speed: %i km/h   Value: %i gold",enemy[i].hp,(int)(enemy[i].speed*GRID),enemy[i].bonus);
	    text->drawtext(specs,surface,x+200,y+2,white,text->tiny);
	    if (enemy[i].aggressive) {
	       snprintf(specs,80,"Damage: %i/second    Range: %.2f",enemy[i].aggressive*FPS,sqrt(enemy[i].range));
	       text->drawtext(specs,surface,x+300,y+GRID/2+2,white,text->tiny);
	    }
        }
    } else {
        SDL_Rect re;
        re.x = 640;
        re.y = 0;
        re.w = 160;
        re.h = 80;
        SDL_FillRect(surface,&re,SDL_MapRGB(surface->format,0,0,0));
        char msg[80];
        snprintf(msg,80,"Gold: %i",gold);
        text->drawtext_c(msg,surface,720,10,white,text->small);
        if (currwave < level->numwaves)
          snprintf(msg,80,"Wave: %i/%i",currwave+1,level->numwaves);
        else
          snprintf(msg,80,"Last wave");
        text->drawtext_c(msg,surface,720,30,white,text->small);
        if (speed>1) {
          snprintf(msg,80,"Speed: %ix",speed);
          text->drawtext_c(msg,surface,720,50,white,text->small);
        }
        redraw_baseinfo = false;
    }
}
void Game::draw_board(SDL_Surface *surface, Text *text) {
  if (paused) {
  	if (redraw_base) {
          boxRGBA(surface,SCREENWIDTH,0,0,SCREENHEIGHT,0,0,0,128);
          text->drawtext_cc("Game paused",surface,400,220,white,text->huge);
          text->drawtext_cc("(press P to unpause)",surface,400,280,white,text->big);
          SDL_Flip(surface);
  	}
  	redraw_base=false;
  	return;
  }
        if (redraw_base) draw_base(base,text);
        if (redraw_baseinfo && levelnb>0) draw_baseinfo(base,text);
        int r=SDL_BlitSurface(base,NULL,surface,NULL);
        if (r == -2) throw Exiter("damn");
        if (r == -1) throw Exiter("huh");

        SDL_Rect re;
        re.w = GRID;
        re.h = GRID;
        re.x = level->x+sc*GRID;
        re.y = level->y+sr*GRID;
        if (levelnb > 0) {
          draw_selection(re,surface);
          draw_info(surface,level->m(sr,sc),text);
	}
        for (unsigned int i=0; i<enemy.size(); i++) {
            re.x = level->x+enemy[i].c*GRID + enemy[i].offset*enemy[i].dc;
            re.y = level->y+enemy[i].r*GRID + enemy[i].offset*enemy[i].dr;
            draw_enemy(&re,surface,enemy[i]);
        }
        for (unsigned int i=0; i<effects.size(); i++) {
            draw_effect(surface,effects[i]);
        }
        SDL_Flip(surface);
}

void go_dir(Thing &t, int d) {
    t.dr = 0;
    t.dc = d;
    t.sdr = 0;
    t.sdc = d;
    t.odr = 0;
    t.odc = d;
    t.osdr = 0;
    t.osdc = d;
}

void Game::init_help() {
    int r = 0;
    int c = 1;
    for (r=0; r<13; r++) go_dir(level->m(r,c),2);
    c = 3;
    for (r=0; r<13; r++) go_dir(level->m(r,c),2);
    c = 5;
    for (r=0; r<13; r++) go_dir(level->m(r,c),-1);
    c = 4;
    for (r=0; r<13; r++) go_dir(level->m(r,c),-2);
    c = 2;
    for (r=0; r<13; r++) go_dir(level->m(r,c),-1);
    c = 1;
    r = 1;
    while(1) {
	Enemy ce = level->wave[currwave*level->sizewave + currenemy++];
	if (ce.type == '*' || currenemy > level->sizewave) {
	    break;
	} else {
	    ce.init(r++,c);
	    choosedir(ce);
	    enemy.push_back(ce);
	}
    }
    level->x = 0;
    level->y = 0;
}
void Game::init() {
    enemy = std::vector<Enemy>();
    effects = std::vector<Effect>();
    clock = 0;
    currwave = 0;
    currenemy = 0;
    sr = sc = 0;
    pleft = 8;
    redraw_base = true;
    redraw_baseinfo = true;
    can_build_cache = false;
    can_build_cache_r = -1;
    can_build_cache_c = -1;
    speed=1;
    paused=false;
}
void Game::loadlevel(int nb, Sound *sound) {
    if (level) level->~Level();
    level = new Level(nb);
    init();
    gold /= 4;
    gold += level->initgold;
    pleft = level->initp;
    sr = level->rows/2;
    sc = level->cols/2;
    if (sound) sound->playmusic(level->bgmusic);
    levelnb = nb;
    if (levelnb == -1) init_help();
    printf("Loading images...\n");
    IMG_Init(IMG_INIT_PNG);
    SDL_Surface *img;
    img=IMG_Load(level->tiles);
    SDL_FreeSurface(tiles);
    tiles=SDL_DisplayFormat(img);
    SDL_FreeSurface(img);
}
Game::Game() {
    level = NULL;
    init();
    show_paths=false;
    gold=0;
    printf("Loading images...\n");
    IMG_Init(IMG_INIT_PNG);
    SDL_Surface *img;
    img=IMG_Load("data/tiles.png");
    tiles=SDL_DisplayFormat(img);
    SDL_FreeSurface(img);
    img=IMG_Load("data/bg.png");
    basebg=SDL_DisplayFormat(img);
    base=SDL_DisplayFormat(img);
    SDL_FreeSurface(img);
    IMG_Quit();

    if(!tiles) throw Exiter("ERROR: could not open tiles");
    if(!basebg) throw Exiter("ERROR: could not open background");
    if(!base) throw Exiter("ERROR: strange (not enough memory?)");
}

void Game::choosedir(Enemy &e) {
    if (e.unpredictable && levelnb != -1) {
	if (rand()%100 < e.unpredictable) {
	    bool okdir=false;
	    while(!okdir) {
		e.dr = rand()%3 - 1;
		e.dc = rand()%3 - 1;
		if (e.dr == 0 && e.dc == 0) continue;
		if (!level->canwalk(e.swimmer,e.r+e.dr,e.c+e.dc)
		 || !level->canwalk(e.swimmer,e.r,e.c+e.dc)
		 || !level->canwalk(e.swimmer,e.r+e.dr,e.c)) continue;
		okdir = true;
	    }
	    return;
	}
    }
    if (e.berserker || e.flyer) {
      if (e.swimmer) {
	e.dr = level->m(e.r,e.c).osdr;
	e.dc = level->m(e.r,e.c).osdc;
      } else {
	e.dr = level->m(e.r,e.c).odr;
	e.dc = level->m(e.r,e.c).odc;
      }
      if (!e.flyer && !level->canwalk(e.swimmer,e.r+e.dr,e.c+e.dc)) {
        e.dr = e.dc = 0;
      }
      return;
    }
    if (e.swimmer) {
	e.dr = level->m(e.r,e.c).sdr;
	e.dc = level->m(e.r,e.c).sdc;
    } else {
	e.dr = level->m(e.r,e.c).dr;
	e.dc = level->m(e.r,e.c).dc;
    }
}
bool Game::in_range(Thing &t, Enemy &e, float range) {
    float rdiff = (t.r - (e.r + e.dr*e.offset/GRID));
    float cdiff = (t.c - (e.c + e.dc*e.offset/GRID));
    float sqdist = rdiff*rdiff + cdiff*cdiff;
    return sqdist < range;
}
int Game::find_target(Thing &t, Sound *sound) {
    int target = -1;
    for (unsigned int i=0; i<enemy.size(); i++) {
	if (in_range(t,enemy[i],t.range())) {
	    if (t.mode == 0 || t.mode == 1) {
		target = i;
		break;
	    } else if (t.mode == 2) {
		if (target == -1 || enemy[i].hp < enemy[target].hp)
		    target = i;
	    } else if (t.mode == 3) {
		if (target == -1 || enemy[i].hp > enemy[target].hp)
		    target = i;
	    } else if (t.mode == 4) {
		if (target == -1 || enemy[i].speed > enemy[target].speed)
		    target = i;
	    } else if (t.mode == 5) {
		if (target == -1 || enemy[i].aggressive > enemy[target].aggressive)
		    target = i;
	    }
	}
    }

    if (target != t.target && target != -1)
        if (sound) sound->playsound(3);

    t.target = target;
    return target;
}
int Game::do_step(Sound *sound) {
  if (paused) {
  	return 0;
  }
  effects.clear();
  for(int times=0; times<speed; times++) {
  clock++;
  if (levelnb != -1) {
  // spawn enemies
  if (currwave < level->numwaves) {
    // float spawnspeed= 1.0 + (float)currwave/level->numwaves;
//    if (clock % ((int)(GRID*2 / spawnspeed)) == 0) {
    if (clock % (50-currwave) == 0) {
	Enemy ce = level->wave[currwave*level->sizewave + currenemy++];
	if (ce.type == '*' || currenemy > level->sizewave) {
	    currwave++;
	    currenemy=0;
	    redraw_baseinfo=true;
	} else if(ce.type != '-') { // use - to have pauses in a wave
	    int pos = 0;
	    if (currwave < level->nb_e_locs) pos = currwave;
	    else if (1+currwave < 2*level->nb_e_locs) pos = (currwave + currenemy) % (2 + currwave - level->nb_e_locs);
	    else pos = (3*currwave + 7*currenemy) % (level->nb_e_locs);
	    ce.init(level->e_initr[pos],level->e_initc[pos]);
	    choosedir(ce);
	    float hpscale = 1.0 + currwave/5.0;
	    ce.hp *= hpscale;
	    ce.maxhp *= hpscale;
	    enemy.push_back(ce);
	    if (sound) sound->playsound(0);
	}
    }
  } else {
    if (enemy.empty()) {
	return 1; // level done
    }
  }
  
  // let towers shoot
  if (clock % 1 == 0) {
	for(unsigned int i=0; i<level->map.size(); i++) {
	    if (level->map[i].istower() && level->map[i].range()) {
		int t=level->map[i].target;
		if (t == -1 || t >= (int)enemy.size()
		    || !in_range(level->map[i],enemy[t],level->map[i].range())
		    || level->map[i].mode > 1) {
		    t = find_target(level->map[i],sound);
		}
		if (t == -1) continue;
		if (clock % level->map[i].bs() == level->map[i].bs()-1) {
		    enemy[t].hp -= level->map[i].bs() * level->map[i].dmg();
		    if (sound && level->map[i].bs()>1) sound->playsound(1);
		}
		Effect e;
		e.type = level->map[i].type;
		e.x1 = level->map[i].c*GRID + level->x + GRID/2;
		e.y1 = level->map[i].r*GRID + level->y + GRID/2;
		e.x2 = enemy[t].c*GRID + level->x + GRID/2 + enemy[t].dc*enemy[t].offset;
		e.y2 = enemy[t].r*GRID + level->y + GRID/2 + enemy[t].dr*enemy[t].offset;
		e.size = 2+level->map[i].dmg();
		e.dmg = level->map[i].dmg();
		e.bs = level->map[i].bs();
		effects.push_back(e);
	    }
	}
  }
  }
  // move enemies
  if (clock % 1 == 0)
      for (unsigned int i=0; i<enemy.size(); i++) {
        if (enemy[i].hp <= 0) {
	    gold += enemy[i].bonus;
	    redraw_baseinfo = true;
	    enemy.erase(enemy.begin()+i);
	    if (sound) sound->playsound(2);
	    i--;
	    continue;
        }
        
        // shoot
        if (enemy[i].aggressive && levelnb != -1) {
	    for(unsigned int j=0; j<level->map.size(); j++) {
		if (level->map[j].istower() && in_range(level->map[j],enemy[i],enemy[i].range)) {
		    if (clock%10 == 9) {
			level->map[j].hp -= 10*enemy[i].aggressive;
			sound->playsound(4);
		    }
		    Effect e;
		    e.type = 'E';
		    if (enemy[i].berserker) e.type = 'B';
		    e.x1 = enemy[i].c*GRID + level->x + GRID/2 + enemy[i].dc*enemy[i].offset;
		    e.y1 = enemy[i].r*GRID + level->y + GRID/2 + enemy[i].dr*enemy[i].offset;
		    e.x2 = level->map[j].c*GRID + level->x + GRID/2;
		    e.y2 = level->map[j].r*GRID + level->y + GRID/2;
		    e.size = sqrt(enemy[i].range)*GRID;
		    e.dmg = enemy[i].aggressive;
		    e.bs = 10;
		    effects.push_back(e);
		    if (level->map[j].hp < 0) {
			level->map[j].type = ' ';
			level->compute_paths();
			redraw_base = true;
			if (sound) sound->playsound(5);
		    }
		    if (!enemy[i].berserker) break;
		}
	    }
        }
        // move
	if (enemy[i].dr && enemy[i].dc) {
	    if (enemy[i].dr == 2 || enemy[i].dc == 2 || enemy[i].dr == -2 || enemy[i].dc == -2) {
		enemy[i].offset += enemy[i].speed*0.4472135955;
	    } else {
		enemy[i].offset += enemy[i].speed*0.70710678;
	    }
	} else {
	    if (enemy[i].dr == 2 || enemy[i].dc == 2 || enemy[i].dr == -2 || enemy[i].dc == -2) {
		enemy[i].offset += enemy[i].speed*0.5;
	    } else {
		enemy[i].offset += enemy[i].speed;
	    }
	}
	if (enemy[i].offset > GRID || (!enemy[i].dr && !enemy[i].dc)) {
	    enemy[i].offset = 0;
	    enemy[i].r += enemy[i].dr;
	    enemy[i].c += enemy[i].dc;
	    choosedir(enemy[i]);
	    if (enemy[i].r == level->destr && enemy[i].c == level->destc) {
		if (--pleft < 1) {
		    return -1; // game over
		}
		enemy[i].hp = 0; // kill it
		gold -= enemy[i].bonus*3/2;
		redraw_base = true;
	    }
	}
      }
    }
    return 0;
}

bool Game::unreachables(bool swimmer) {
    for (int i=0; i<level->nb_e_locs; i++)
	if (level->m(level->e_initr[i],level->e_initc[i]).dist == INFDIST) return true;
    for (unsigned int i=0; i<enemy.size(); i++)
	if (!enemy[i].flyer && !enemy[i].berserker && enemy[i].swimmer == swimmer && level->m(enemy[i].r,enemy[i].c).dist == INFDIST ) return true;
    return false;
}
bool Game::canbuild() {
    if (levelnb <= 0) return false;
    Thing &t = level->m(sr,sc);
    if (t.type != ' ') return false;
    if (sr == can_build_cache_r && sc == can_build_cache_c) {
	return can_build_cache;
    }
    bool cb = false;
    if (t.type == ' ') {
	cb = true;
	t.type = 'T';
	level->compute_paths(false);
	if (unreachables(false)) cb = false;
	if (cb) {
	  level->compute_paths(true);
	  if (unreachables(true)) cb = false;
	}
	t.type = ' ';
	level->compute_paths(true);
	level->compute_paths(false);
    }
    can_build_cache_c = sc;
    can_build_cache_r = sr;
    can_build_cache = cb;
    return cb;
}
void Game::changemode(int mc) {
    Thing &t = level->m(sr,sc);
    if (t.istower()) {
	t.mode = (t.mode+mc+6) % 6;
    }
}
void Game::destroy() {
    Thing &t = level->m(sr,sc);
    if (t.istower()) {
	t.type = ' ';
	gold += t.sell_value();
	level->compute_paths();
    }
}
void Game::heal() {
    Thing &t = level->m(sr,sc);
    if (t.istower() && t.hp < t.initial_hp()) {
	t.hp += t.initial_hp()/10;
	if (t.hp > t.initial_hp()) t.hp = t.initial_hp();
	gold -= t.sell_value()/10;
    }
}
void Game::build(char what) {
    Thing &t = level->m(sr,sc);
    if (t.type == ' ' && gold >= 50) {
	t.type = what;
	level->compute_paths(false);
	if (unreachables(false)) {
	    t.type = ' ';
	    level->compute_paths(false);
	    return;
	}
	level->compute_paths(true);
	if (unreachables(true)) {
	    t.type = ' ';
	    level->compute_paths(true);
	    level->compute_paths(false);
	    return;
	}
	gold -= 50;
	t.hp = t.initial_hp();
	redraw_base = true;
	for (unsigned int i=0; i<enemy.size(); i++)
	    choosedir(enemy[i]);
	return;
    }
    if (t.can_upgrade() && gold >= t.upgrade_cost()) {
	gold -= t.upgrade_cost();
	t.type = t.upgrade();
	t.hp = t.initial_hp();
	redraw_base = true;
    }
}

void Game::pause() {
	paused = true;
	redraw_base = true;
}

void Game::move(SDLKey k, Sound *sound) {
  if (paused) {
  	if (k == SDLK_p) {
  		paused = false;
		sound->playmusic(level->bgmusic);
  	}
  } else {
    switch(k) {
	case SDLK_RIGHT: sc++; break;
	case SDLK_LEFT: sc--; break;
	case SDLK_UP: sr--; break;
	case SDLK_DOWN: sr++; break;
	case SDLK_HOME: build('B'); break;
	case SDLK_END: build('T'); break;
	case SDLK_PAGEUP: heal(); break;
	case SDLK_PAGEDOWN: destroy(); break;
	case SDLK_RSHIFT: changemode(-1); break;
	case SDLK_RCTRL: changemode(1); break;
	case SDLK_SPACE: show_paths = !show_paths; redraw_base=true; break;
	case SDLK_s: speed *= 3; if (speed > 9) speed=1; redraw_baseinfo=true; break;
	case SDLK_p: pause(); sound->playmusic("data/music/menu.mp3"); break;
	default: break;
    }
    if (sc < 0) sc = 0;
    if (sr < 0) sr = 0;
    if (sc >= level->cols) sc = level->cols-1;
    if (sr >= level->rows) sr = level->rows-1;
  }
}
