#ifndef BOARD_H
#define BOARD_H
#include <vector>
#include <queue>
#include <math.h>
#include <stdlib.h>

#include "SDL.h"
#include "SDL_image.h"
#include "SDL_gfxPrimitives.h"

#include "util.h"
#include "text.h"
#include "enemy.h"
#include "level.h"
#include "sound.h"






struct Effect {
    int type;
    int x1, x2, y1, y2;
    int size;
    int bs;
    int dmg;
};

class Game {
public:
    Level *level;
    std::vector<Enemy> enemy;
    int clock;
    int currwave;
    int currenemy;
    std::vector<Enemy> wave;
    std::vector<Effect> effects;
    int sr,sc;
    int pleft;
    int gold;
    bool show_paths;
    SDL_Surface *tiles;
    SDL_Surface *basebg;
    SDL_Surface *base;

    Game();
    void loadlevel(int nb, Sound *sound);
    void draw_board(SDL_Surface *surface, Text *text);
    int do_step(Sound *sound);
    void move(SDLKey k, Sound *sound);
    int speed;
    bool paused;
    int levelnb;


private:
    bool redraw_base;
    bool redraw_baseinfo;
    bool can_build_cache;
    int can_build_cache_c;
    int can_build_cache_r;
    void init();
    void init_help();
    void draw_base(SDL_Surface *surface, Text *text);
    void draw_baseinfo(SDL_Surface *surface, Text *text);
    void draw_thing(SDL_Rect *re, SDL_Surface *surface, const Thing &t, Text *text);
    void draw_selection(SDL_Rect &re, SDL_Surface *surface);
    void draw_effect(SDL_Surface *surface, const Effect &e);
    void draw_enemy(SDL_Rect *re, SDL_Surface *surface, const Enemy &e);
    void draw_info(SDL_Surface *surface, const Thing &t, Text *text);
    void draw_path(SDL_Surface *surface, int r, int c, bool swimmer, bool o);
    void build(char what);
    void heal();
    void destroy();
    void changemode(int mc);
    void choosedir(Enemy &e);
    void pause();
    bool in_range(Thing &t, Enemy &e, float range);
    int find_target(Thing &t, Sound *sound);
    bool unreachables(bool swimmer);
    bool canbuild();


};

#endif