#include "level.h"

Level::Level(int nb) {
	char lvlfilename[80];
	snprintf(lvlfilename,80,"levels/level%i",nb);
	FILE *fp = fopen(lvlfilename, "r");
	if (!fp) throw Exiter("Level map not found!");
	printf("Loading level %i\n",nb);
	char buff[80];
	if (!feof(fp)) {
		fgets(title, 80, fp);
		for (int i=0; i<80; i++) {
		    if (title[i]=='\n') title[i]=0;
		    if (title[i]==0) break;
		}
		fgets(buff, 80, fp);
		initgold = strtol(buff,NULL,10);
		fgets(buff, 80, fp);
		initp = strtol(buff,NULL,10);
		fgets(buff, 80, fp);
		rows = strtol(buff,NULL,10);
		fgets(buff, 80, fp);
		cols = strtol(buff,NULL,10);
	}
        map = std::vector<Thing>(rows*cols, Thing());
        w = cols*GRID;
        h = rows*GRID;
        x = (SCREENWIDTH-INFOWIDTH)/2 - w/2;
        y = SCREENHEIGHT/2 - h/2 + 5; // + 5 for the title
	nb_e_locs = 0;
	for(int r=0; r<rows; r++) {
		if (feof(fp)) throw Exiter("Level map loading error!");
		fgets(buff, 80, fp);
		for(int c=0; c<cols; c++) {
			m(r,c).type=buff[c];
			m(r,c).r=r;
			m(r,c).c=c;
			if (buff[c] == '*') {destr=r; destc=c;}
			if (buff[c] == 'E') {e_initr[nb_e_locs]=r; e_initc[nb_e_locs++]=c;}
		}
	}
	if (!feof(fp)) {
		fgets(bgmusic, 200, fp);
		for (int i=0; i<200; i++) {
		    if (bgmusic[i]=='\n') bgmusic[i]=0;
		    if (bgmusic[i]==0) break;
		}
		fgets(tiles, 200, fp);
		for (int i=0; i<200; i++) {
		    if (tiles[i]=='\n') tiles[i]=0;
		    if (tiles[i]==0) break;
		}
		
		fgets(buff, 80, fp);
		numwaves = strtol(buff,NULL,10);
		fgets(buff, 80, fp);
		sizewave = strtol(buff,NULL,10);
	}
        wave = std::vector<Enemy>(numwaves*sizewave, Enemy());
	for(int i=0; i<numwaves; i++) {
		if (feof(fp)) throw Exiter("Level map loading error!");
		fgets(buff, 80, fp);
		for(int j=0; j<sizewave; j++) {
			wave[i*sizewave+j].type=buff[j];
			if (buff[j]=='*') break;
		}
	}
	
	fclose(fp);
	if (nb != -1) {
	printf("Level loaded, computing paths...\n");
	compute_paths();
	for(unsigned int i=0; i<map.size(); i++) {
	    map[i].odr = map[i].dr;
	    map[i].odc = map[i].dc;
	    map[i].osdr = map[i].sdr;
	    map[i].osdc = map[i].sdc;
	}
	}
}

bool operator< (const Thing& lhs, const Thing& rhs) {
	return lhs.dist > rhs.dist;
}

#define SQRT2 1.41421356
#define SQRT5 2.2360679785
// clockwise from north + 8 extra dirs
#define NBDIRS 16
static const int nb_dr[NBDIRS] = {-1,-1,0,1,1,1,0,-1,-2,-1,1,2,2,1,-1,-2};
static const int nb_dc[NBDIRS] = {0,1,1,1,0,-1,-1,-1,1,2,2,1,-1,-2,-2,-1};
static const float nb_d[NBDIRS] = {1,SQRT2,1,SQRT2,1,SQRT2,1,SQRT2,SQRT5,SQRT5,SQRT5,SQRT5,SQRT5,SQRT5,SQRT5,SQRT5};

void Level::compute_paths() {
    compute_paths(true);
    compute_paths(false);
}
void Level::compute_paths(bool swimmer) {
    for(unsigned int i=0; i<map.size(); i++) map[i].dist = INFDIST;

    int r=destr;
    int c=destc;
    Thing t;
    t =  m(r,c);
    t.dist = 0;
    std::priority_queue<Thing> q;
    q.push(t);
    while(!q.empty()) {
	t = q.top();
	q.pop();
	r = t.r;
	c = t.c;
	if (t.dist > m(t.r,t.c).dist) continue;
	for (int i=0; i<NBDIRS; i++) {
	    int nr = r+nb_dr[i];
	    int nc = c+nb_dc[i];
	    if (nr < 0 || nc < 0 || nr >= rows || nc >= cols) continue;
	    float ndist = t.dist + nb_d[i];
	    if (ndist >= m(nr,nc).dist) continue;
	    if (!canwalk(swimmer,nr,nc)) continue;
	    if (nb_dr[i] && nb_dc[i]) { // diagonal: need free space
		if (!canwalk(swimmer,r,nc) || !canwalk(swimmer,nr,c)) continue;
		if (!canwalk(swimmer,r,(nc+c)/2) || !canwalk(swimmer,(nr+r)/2,c)) continue;
		if (!canwalk(swimmer,nr,(nc+c)/2) || !canwalk(swimmer,(nr+r)/2,nc)) continue;
	    }
	    m(nr,nc).dist = ndist;
	    if (swimmer) {
	      m(nr,nc).sdr = -nb_dr[i];
	      m(nr,nc).sdc = -nb_dc[i];
	    } else {
	      m(nr,nc).dr = -nb_dr[i];
	      m(nr,nc).dc = -nb_dc[i];
	    }
	    q.push(m(nr,nc));
	}
    }
}


