#ifndef LEVEL_H
#define LEVEL_H
#include <vector>
#include <queue>
#include <stdlib.h>
#include "util.h"
#include "enemy.h"


#define GRID 32
// max nb of enemylocs
#define NB_ENEMYLOCS 8
#define INFDIST 100000

#define SCREENWIDTH 800
#define SCREENHEIGHT 480
#define INFOWIDTH 160


class Thing {
public:
    int dr, dc; // direction towards destination
    int sdr, sdc; // direction towards destination for swimmers
    int odr, odc; // direction towards destination (without towers)
    int osdr, osdc; // direction towards destination for swimmers (without towers)
    float dist; // distance to destination (only used during path computation)

    int r,c;  // my location (redunant)
    int hp;
    int target;  // for towers
    int mode;

    char type;


    Thing() {type=' '; dr=dc=sdr=sdc=0; dist=0; r=c=0; target=-1; mode=0; hp=100;}
    const bool canwalk() const {
	switch (type) {
	    case ' ':
	    case 'E':
	    case '*': return true;
	    default: return false;
	}
    }
    const bool canswim() const {
	switch (type) {
	    case '.': return true;
	    default: return false;
	}
    }
    const bool istower() const {
	switch (type) {
	    case 'B':
	    case 'T':
	    case 'U':
	    case 'V':
	    case 'X':
	    case 'Y': return true;
	    default: return false;
	}
    }
    const int initial_hp() const {
	switch (type) {
	    case 'B': return 5000;
	    case 'T': return 1000;
	    case 'U': return 2000;
	    case 'V': return 3000;
	    case 'Y': return 5000;
	    case 'X': return 8000;
	    default: return false;
	}
    }
    const char upgrade() const {
	switch (type) {
	    case 'T': return 'U';
	    case 'U': return 'V';
	    case 'V': return 'Y';
	    case 'Y': return 'X';
	    default: return ' ';
	}
    }
    const bool can_upgrade() const {
	switch (type) {
	    case 'T':
	    case 'U':
	    case 'V':
	    case 'Y': return true;
	    default: return false;
	}
    }
    const float range() const {
	float r=0.0;
	switch (type) {
	    case 'T': r = 5; break;
	    case 'U': r = 7; break;
	    case 'V': r = 10; break;
	    case 'Y': r = 15; break;
	    case 'X': r = 20; break;
	    default: r = 0;
	}
	if (mode==1) r *= 2;
	return r;
    }
    const int dmg() const {
	int d=0;
	switch (type) {
	    case 'T': d = 2; break;
	    case 'U': d = 4; break;
	    case 'V': d = 6; break;
	    case 'Y': d = 8; break;
	    case 'X': d = 10; break;
	    default: d = 0;
	}
	if (mode==1) d /= 2;
	return d;
    }
    const int bs() const {
	int d=0;
	switch (type) {
	    case 'T': d = 14; break;
	    case 'U': d = 13; break;
	    case 'V': d = 11; break;
	    case 'Y': d = 1; break;
	    case 'X': d = 1; break;
	    default: d = 1;
	}
	return d;
    }
    const int upgrade_cost() const {
	switch (type) {
	    case 'T': return 100;
	    case 'U': return 150;
	    case 'V': return 200;
	    case 'Y': return 300;
	    default: return 0;
	}
    }
    const int sell_value() const {
	switch (type) {
	    case 'B': return 40*hp/initial_hp();
	    case 'T': return 25;
	    case 'U': return 75;
	    case 'V': return 150;
	    case 'Y': return 250;
	    case 'X': return 400;
	    default: return 0;
	}
    }
    const char* description(int line) const {
      if (line==1) {
	switch (type) {
	    case 'W': return "Wall";
	    case 'B': return "Obstacle";
	    case 'T': return "Basic Shooter";
	    case 'U': return "Shooter";
	    case 'V': return "Mega Shooter";
	    case 'Y': return "Laser";
	    case 'X': return "Mega Laser";
	    case ' ': return "Empty space";
	    case '.': return "Water";
	    case 'E': return "Enemy spawn";
	    case '*': return "Lives left";
	    default: return "unknown";
	}
      } else {
	switch (type) {
	    case 'W': return "Blocks all enemies";
	    case 'B': return "Blocks all non-flyers";
	    case 'T': return "Shoots little bullets";
	    case 'U': return "Shoots medium bullets";
	    case 'V': return "Shoots big bullets";
	    case 'Y': return "Medium laser";
	    case 'X': return "Strong laser";
	    case ' ': return "You can build towers here";
	    case '.': return "Only swimmers can pass";
	    case 'E': return "Enemies appear here";
	    case '*': return "You lose if it reaches zero";
	    default: return "unknown";
	}
      }
    };
};


class Level {
public:
    int rows;
    int cols;
    std::vector<Thing> map;
    int x,y; // position on the screen
    int w,h; // size of level
    int destr, destc;  // location of destination
    int nb_e_locs;
    int e_initr[NB_ENEMYLOCS];
    int e_initc[NB_ENEMYLOCS];
    std::vector<Enemy> wave;
    int sizewave;
    int numwaves;
    int initgold;
    int initp;
    char title[80];
    char bgmusic[200];
    char tiles[200];

    Level(int nb);
    void compute_paths();
    Thing& operator()(int r, int c) {
//        if (r >= rows || r < 0 || c >= cols || c < 0) return NULL;
        return map[r*cols + c];
    }
    Thing& m(int r, int c) {
//        if (r >= rows || r < 0 || c >= cols || c < 0) return NULL;
        return map[r*cols + c];
    }
    void compute_paths(bool swimmer);
    bool canwalk(bool canswim, int r, int c) {
	if (r < 0 || c < 0) return false;
	if (r >= rows || c >= cols) return false;
        return m(r,c).canwalk() || (canswim && m(r,c).canswim());
    }
};




#endif