/*
    Pandora tower
    Copyright (C) 2012  Jon Sneyers

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>

#include "SDL.h"

#include "util.h"
#include "text.h"
#include "board.h"
#include "sound.h"


const Uint32 delay = 1000/FPS;


SDL_Surface *Screen = NULL;
Text *text;
Game *game;
Sound *sound;

Game *help;
bool showhelp;


void do_key(SDLKey k, int onoff) {
	switch(k) {
	    case SDLK_RIGHT:
	    case SDLK_LEFT:
	    case SDLK_UP:
	    case SDLK_DOWN:
	    case SDLK_HOME:
	    case SDLK_END:
	    case SDLK_PAGEUP:
	    case SDLK_PAGEDOWN:
	    case SDLK_RSHIFT:
	    case SDLK_RCTRL:
	    case SDLK_SPACE:
	    case SDLK_p:
	    case SDLK_s:
		if(onoff) game->move(k,sound);
		break;
	    case SDLK_h:
		if (onoff) showhelp = !showhelp;
		break;
	    case SDLK_ESCAPE: throw Exiter("Pressed escape");
	    default:
//		printf("Unknown key: %i\n",k);
		return;
	}
}
void mainloop() {
  SDL_Event event;
  Uint32 ticks;
  int lvlnb = 1;
  int frames = 0;
  int d = 0;
  Uint32 beginticks = SDL_GetTicks();
  while(1) {
    ticks = SDL_GetTicks();
    while(SDL_PollEvent(&event)) {
	switch (event.type) {
		case SDL_KEYDOWN:
			do_key(event.key.keysym.sym,1);
			break;
		case SDL_KEYUP:
			do_key(event.key.keysym.sym,0);
			break;
		case SDL_QUIT:
			throw Exiter("Close button of window clicked");
	}
    }

    if (showhelp) {
	help->do_step(NULL);
	help->draw_board(Screen,text);
    } else {
      int done = game->do_step(sound);
      if (done) {
	if (done == 1) game->loadlevel(++lvlnb, sound);
	else throw Exiter("Game over!");
      }
      game->draw_board(Screen,text);
    }


    d = delay - (SDL_GetTicks()-ticks); // + (d<0 ? d : 0);

    frames++;
    ticks = SDL_GetTicks();
    if (frames%100==0) printf("frames:%i in %u ms (%.2f FPS, aim=%i)\n",frames,ticks-beginticks,1000.0*frames/(ticks-beginticks),FPS);
//    printf("%i/%i\n",d,delay);
    if (d>0) SDL_Delay(d);
  }
}



int main(int argc, char **argv) {

	const bool Fullscreen = true;

	printf("Initializing SDL...");
	if ( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) < 0 )
	{
		fprintf(stderr, "Error initializing SDL: %s\n", SDL_GetError());
		return 1;
	}

	try {
		printf("OK.\n");
		
		SDL_WM_SetCaption("Pandora tower", NULL);  // probably never shown
		printf("Setting video mode: 800x480x32...");
		Uint32 flags = SDL_HWSURFACE | SDL_DOUBLEBUF;
		if (Fullscreen)
			flags |= SDL_FULLSCREEN;
		Screen = SDL_SetVideoMode(800, 480, 32, flags);
		if ( Screen == NULL )
			throw Exiter(SDL_GetError());
		else
			printf("OK\n");

		if (Fullscreen)				// hide mouse cursor in fullscreen
			SDL_ShowCursor(SDL_DISABLE);

		srand(time(NULL));

		sound = new Sound();
		text = new Text();
		game = new Game();
		help = new Game();

		game->loadlevel(1,sound);
		help->loadlevel(-1,NULL);
		showhelp = false;

		mainloop();
	}
	catch(Exiter &e)
	{
		printf("\nExiting: %s\n", e.message.c_str());
	}
	if (sound) sound->exit();
	SDL_Quit();
	return 0;
}

